/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.attachments;

import java.io.ByteArrayOutputStream;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.DOMReader;
/**
 * 
 * @author andrea zoppello
 * The  AttachmentsToMessageContent takes the attachmnents in the normalized message
 * and put into the message content encoded in base 64
 * 
 */
public class AttachmentsToMessageContent extends AttachmentTransformComponentSupport implements
	MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(AttachmentsToMessageContent.class);

	
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws Exception {
		
		SourceTransformer sourceTransformer = new SourceTransformer();
    	
		try {
    		log.debug("AttachmentsToMessageContent");
    		DOMReader xmlReader = new DOMReader();
    		
    		org.w3c.dom.Document inputAsDOM = sourceTransformer.toDOMDocument(in);
    		Document inputDocument = xmlReader.read(inputAsDOM);
    	    Element rootElement = inputDocument.getRootElement();
    		Set<String> attachmentNames = in.getAttachmentNames();
    		DataHandler dh = null;
    		String base64String = null;
    		ByteArrayOutputStream aByteArrayOutputStream = null;
    		
    		for (String s : attachmentNames){
    			
    			dh = in.getAttachment(s);
    			dh.getContentType();
    			
    			aByteArrayOutputStream = new ByteArrayOutputStream();
    			dh.writeTo(aByteArrayOutputStream);
    			byte[] buff = aByteArrayOutputStream.toByteArray();
    			
    			base64String = new String(new Base64().encode(buff)); 
    			
    			rootElement.addElement(getBase64attachmentElementName())
                .addAttribute( "name", s )
                .addAttribute( "content-type", dh.getContentType())
                .addText(base64String);	
    		
    		}
    		
    		
    		
    		out.setContent(new StringSource(inputDocument.asXML()));
    		
    		return true;
		
		}catch(Exception e){
    		e.printStackTrace();
    		log.error("Exception AttachmentsToMessageContent");
    		log.error(e);
    		Fault fault = exchange.createFault();
        	fault.setContent(new StringSource("<ERROR>AttachmentsToMessageContent produced a null message</ERROR>"));
        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
        	throw new FaultException("AttachmentsToMessageContent Failed ", exchange, fault);
    	}
	} 
	
	
	
   

}
