/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.jdbcquery;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.dbutils.ResultSetHandler;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * ResultSetHandler implementation that put the query result in a dom4j
 * Element.
 * 
 * @author buso
 *
 */
/*
 * TODO Propose this code to DBUtils maintainers.
 */
public class XmlResultSetHandler implements ResultSetHandler {
	private String rowsXmlEnvelopName = "rows";
	private String rowXmlEnvelopName = "row";
	
	public XmlResultSetHandler(String rowsXmlEnvelopName, String rowXmlEnvelopName) {
		super();
		if (rowsXmlEnvelopName != null && !rowsXmlEnvelopName.trim().equals("")) {
			this.rowsXmlEnvelopName = rowsXmlEnvelopName;
		}
		if (rowXmlEnvelopName != null && !rowXmlEnvelopName.trim().equals("")) {
			this.rowXmlEnvelopName = rowXmlEnvelopName;
		}
	}

	public XmlResultSetHandler() {}
	
	public Object handle(ResultSet rs) throws SQLException {
		
        ResultSetMetaData rsmd = rs.getMetaData();
        int cols = rsmd.getColumnCount();

        Element rows = DocumentHelper.createElement(rowsXmlEnvelopName);
        Element row = null;
        while (rs.next()) {
    		row = DocumentHelper.createElement(rowXmlEnvelopName);
    		Element column = null;
	        for (int i = 1; i <= cols; i++) {
	        	Object value = rs.getObject(i);
	        	column = DocumentHelper.createElement(rsmd.getColumnName(i));
	        	int columnType = rsmd.getColumnType(i); 
	        	if (columnType == Types.BINARY) {
	        		
	        		column.setText((value != null)?
	        			new String(Base64.encodeBase64((byte[])value)) : "");
	        	} else {
	        		column.setText((value != null)? value.toString() : "");
	        	}
	        	row.add(column);
	        }
	        rows.add(row);
        }
		return rows;
	}

	public String getRowsXmlEnvelopName() {
		return rowsXmlEnvelopName;
	}

	public void setRowsXmlEnvelopName(String rowsXmlEnvelopName) {
		this.rowsXmlEnvelopName = rowsXmlEnvelopName;
	}

	public String getRowXmlEnvelopName() {
		return rowXmlEnvelopName;
	}

	public void setRowXmlEnvelopName(String rowXmlEnvelopName) {
		this.rowXmlEnvelopName = rowXmlEnvelopName;
	}

}
