/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.jdbcstore;

import it.eng.spagosoa.smx.components.jdbcquery.JDBCConnectionConfig;
import it.eng.spagosoa.smx.components.jdbcquery.JDBCQueryConfig;
import it.eng.spagosoa.smx.components.jdbcquery.QueryParameterConfig;
import it.eng.spagosoa.smx.components.jdbcquery.XmlResultSetHandler;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.dbutils.DbUtils;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.util.MessageUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.dom.DOMDocument;

/**
 * Jdbc component that use the input NormalizedMessage to fill the store
 * procedure call statement.
 * <br>
 * The result is optionally inserted into a copy of the incoming message
 * otherwise a new message is create as output. The output should be formed
 * of more results in this case the call statement placeHolders are used
 * to rappresents output values on output message.
 * 
 * @author buso
 *
 */
public class JDBCStoreProcedureComponent extends TransformComponentSupport
	implements MessageExchangeListener {
	
	private JDBCConnectionConfig connConfig;
	private JDBCQueryConfig queryConfig;

	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {

		// transform query in jdbc format
		String jdbcQuery = queryConfig.getQuery().replaceAll(queryConfig.getMatchParamRegexp(),"?");
		
		// retrives parameter from in message
		// input params
		Object[] inParams = queryConfig.getParamsFromInXml
			(in, filterInputParams(queryConfig.getQueryParams(), true));
		// output params
		int[] outParams = getOutParamSqlTypes(filterInputParams(queryConfig.getQueryParams(), false));
		
		// execute the call
			// get the connection
		Object[] results = null;
		Connection conn = connConfig.getConnection();
		try {
			StoreProcedureRunner spRunner = new StoreProcedureRunner();
			results = (Object[])spRunner.call(conn, jdbcQuery, inParams, outParams,
				new XmlResultSetHandler(
					queryConfig.getRowsXmlEnvelope(),
					queryConfig.getRowXmlEnvelope())
			);
		} catch (SQLException sqle) {
			if (JDBCQueryConfig.FAULT_FLOW.equals(queryConfig.getFaultManagement())) {
				Fault fault = exchange.createFault();
					Element faultElem = DocumentHelper.createElement("Fault");
					Element sqlStateElem = DocumentHelper.createElement("sql-state");
					sqlStateElem.setText(sqle.getSQLState());
					Element errorCodeElem = DocumentHelper.createElement("error-code");
					errorCodeElem.setText(""+sqle.getErrorCode());
				faultElem.add(sqlStateElem);
				faultElem.add(errorCodeElem);
				fault.setContent(new StringSource(faultElem.asXML()));
				MessageUtil.transfer(fault, out);
				return true;
			}
			throw sqle;
		} finally {
			DbUtils.close(conn);
		}
		
		// compose out response. you need to manage single values and resultset
		// values (last ones were manages by ResultSetHandler)
		if (queryConfig.isEnrichMessage()
			&& queryConfig.getXmlEnvelope() != null) {
			MessageUtil.transfer(in, out);
			SourceTransformer trans = new SourceTransformer();
			String strContent = trans.contentToString(out);
			Document docContent = DocumentHelper.parseText(strContent);
			Node contentEnvelop = docContent.selectSingleNode(
				"/"+queryConfig.getXmlEnvelope());
			if (queryConfig.isEnrichMessage() && results.length < 2) {
				Element tmpResult = DocumentHelper.createElement("tmpResults");
				getResultsNodes(results, tmpResult);
				((Element)contentEnvelop).appendContent( tmpResult );
			} else {
				getResultsNodes(results, (Element)contentEnvelop);
			}
			out.setContent(new StringSource( docContent.asXML() ));
		} else {
			DOMDocument docContent = new DOMDocument();
			docContent.setRootElement(DocumentHelper.createElement(queryConfig.getXmlEnvelope()));
			getResultsNodes(results, docContent.getRootElement());
			out.setContent(new StringSource(docContent.asXML()));
		}
		
		
		return true;
	}
	
	private int[] getOutParamSqlTypes(List<QueryParameterConfig> queryParamConfigs) {
		int[] sqlTypes = new int[queryParamConfigs.size()];
		QueryParameterConfig config = null;
		Iterator<QueryParameterConfig> iter = queryParamConfigs.iterator();
		for (int idx = 0; iter.hasNext(); idx++) {
			config = iter.next();
			sqlTypes[idx] = Integer.parseInt(config.getParamType());
		}
		return sqlTypes;
	}
	
	/**
	 * Return all results in dom4j elements where the element name is the
	 * placeHolder configured for the output parameter.
	 * 
	 * @param results
	 * @param parent
	 */
	private void getResultsNodes(Object[] results, Element parent) {
		Object elemObj = null;
		for (int i = 0; i < results.length; i++) {
			elemObj = results[i];
			if (elemObj instanceof Element) {
				parent.add((Element)elemObj);
			} else {
				Element elem = DocumentHelper.createElement(
					queryConfig.getQueryParams().get(i).getPlaceHolder()
				);
				elem.setText(elemObj != null? elemObj.toString() : "");
				parent.add(elem);
			}
		}
	}
	
	private List<QueryParameterConfig> filterInputParams(List<QueryParameterConfig> queryParamsCfg,
		boolean returnInput)
	{
		List<QueryParameterConfig> toReturn = new ArrayList<QueryParameterConfig>();
		Iterator<QueryParameterConfig> iter = queryParamsCfg.iterator();
		QueryParameterConfig elem = null;
		while (iter.hasNext()) {
			elem = iter.next();
			if (!elem.isOutputParam() && returnInput) {
				toReturn.add(elem);
			} else if (elem.isOutputParam() && !returnInput) {
				toReturn.add(elem);
			} else {
				logger.warn("Unhandled QueryParamConfig: "+elem);
			}
		}
		
		return toReturn;
	}

	public JDBCConnectionConfig getConnConfig() {
		return connConfig;
	}

	public void setConnConfig(JDBCConnectionConfig connConfig) {
		this.connConfig = connConfig;
	}

	public JDBCQueryConfig getQueryConfig() {
		return queryConfig;
	}

	public void setQueryConfig(JDBCQueryConfig queryConfig) {
		this.queryConfig = queryConfig;
	}

}
