/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.manage;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.apache.servicemix.jbi.jaxp.StringSource;

/**
 * This is a service engines that return the datasources deployed in smx 
 * 
 * @author Andrea Zoppello
 *
 */
public class GetResources extends TransformComponentSupport implements
		MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(GetResources.class);

	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws Exception {
		
		StringBuffer sb = new StringBuffer();
		InitialContext context = getContext().getNamingContext();
		
		sb.append("<DATASOURCES>");
		
		//NamingEnumeration<NameClassPair> ne = context.list("java:comp");
		Context comp = (Context)context.lookup("java:comp");
		Context env = (Context)comp.lookup("env");
		
		try {
			NamingEnumeration<Binding> ne =  env.listBindings("jdbc");
			Binding binding = null;
			DataSource ds  = null;
			while (ne.hasMore()) {
			    binding = ne.next();
			    if (binding.getObject()  instanceof DataSource) {
			    	ds  = (DataSource)binding.getObject();
			    	sb.append("<DATASOURCE name=\""+binding.getName()+"\" " );
			    	if (ds instanceof BasicDataSource) {
			    		sb.append(" driver = \""+((BasicDataSource)ds).getDriverClassName()+"\" " );
			    		sb.append(" user = \""+((BasicDataSource)ds).getUsername()+"\" " );
			    		sb.append(" url = \"" + ((BasicDataSource)ds).getUrl()+"\" " );
			    		sb.append(" password = \"" + ((BasicDataSource)ds).getPassword()+"\" " );
					}
			    	sb.append("></DATASOURCE>");
			    }
			}
		} catch (Exception ex) {
			log.error("Error during retrieval of datasources", ex);
		}
		
		sb.append("</DATASOURCES>");
		out.setContent(new StringSource(sb.toString()));
		return true;
	}
	


	

	

}
