/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.monitoring.jms;

import it.eng.spagosoa.metadb.base.MepStateConstants;
import it.eng.spagosoa.metadb.base.ServiceAssemblyStateConstants;
import it.eng.spagosoa.metadb.dbutils.AuditDBUtils;
import it.eng.spagosoa.metadb.dbutils.HibernateUtil;
import it.eng.spagosoa.metadb.model.Attribute;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.InstanceIter;
import it.eng.spagosoa.metadb.model.InstanceSa;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spagosoa.metadb.model.JournalInstanceService;
import it.eng.spagosoa.metadb.model.LogJournal;
import it.eng.spagosoa.metadb.model.MepState;
import it.eng.spagosoa.metadb.model.RelevantData;
import it.eng.spagosoa.metadb.model.RelevantDataInstanceSa;
import it.eng.spagosoa.metadb.model.RelevantDataInstanceSaId;
import it.eng.spagosoa.metadb.model.Rule;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.spagosoa.metadb.model.XmlNamespace;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jms.JmsEndpoint;
import org.apache.servicemix.jms.standard.StandardConsumerProcessor;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Session;
import org.hibernate.Transaction;

// Details on how to send ANY type of message on a JMS queue for ServiceMix, 
// please read the following link:
// http://www.nabble.com/MapMessage-and-or-ObjectMessage-tf2226788.html#a6170831

public class MonitorStandardConsumerProcessor extends StandardConsumerProcessor {

	public static final String MEP_INONLY = "org.apache.servicemix.jbi.messaging.InOnlyImpl";

	public static final String MEP_INOUT = "org.apache.servicemix.jbi.messaging.InOutImpl";

	public static final String MEP_ROBUSTINONLY = "org.apache.servicemix.jbi.messaging.RobustInOnlyImpl";

	public static final String MEP_INOPTIONALOUT = "org.apache.servicemix.jbi.messaging.InOptionalOutImpl";
	
	public static Map<String, String> namespaceURI = null;
	
	private static final Log log = LogFactory.getLog(MonitorStandardConsumerProcessor.class);

	//	For a consumer endpoint, indicates if the processor
	//    should store the exchanges in the meta database, for
	//    using the restart feature.
	private boolean storeExchangesForRestart = true;
	
	//    For a consumer endpoint, indicates if the processor
	//    should keep, on the database, only the faulted exchanges
	//    (before the fault).
	//    If false, all ACTIVE exchanges are stored in the database,
	//    not only the faulted exchanges.
	private boolean keepOnlyFaultedExchanges = true;
	
	
	public class NewExchangeStatus {
		boolean exchangeDone = false; 	// If the message exchange has the status done, this variable
		  // is setted and used to check if the process is terminated
		boolean processFault = false; 	// If the message exchange has the status fault, set the entire
		  // process state to faulted
		MepState exchangeStatus = null;	// Exchange status to save in log_journal table
	}
	
	public class JISCreationResult {
		JournalInstanceService jis = null;
		boolean instanceSACreated = false;
	}
	

	public MonitorStandardConsumerProcessor(JmsEndpoint ep) throws Exception {
		super(ep);
		loadRestartConfiguration();
	}

	@Override
	public void onMessage(Message message) {
		
		if (!(message instanceof ObjectMessage)) {
			log.warn("The message is not of type ObjectMessage");
			return;	
		}
		
		try {
			MessageExchange exchange = (MessageExchange)((ObjectMessage)message).getObject();
	        
	    	debug("traceExchange. Exchange = " + exchange);
	    	traceExchange(exchange);
		}
		catch (JMSException ex) {
			log.error("Unable to extract Exchange from message", ex);
		}
	}


	
	public void debug(Object o){
		if (log.isDebugEnabled()){
			log.debug(o);
		}
	}
	
	public static Map<String, String> getNamespaceURI(){
		if (namespaceURI == null)
			initNameSpaceURI();
		return namespaceURI;
	}
	
	public synchronized static void initNameSpaceURI(){
		Session aSession = null;
		try {
			namespaceURI = new HashMap<String, String>();
			aSession = HibernateUtil.getSessionFactory().openSession();
			List<XmlNamespace> lNs = aSession.createQuery(" from it.eng.spagosoa.metadb.model.XmlNamespace").list();
			for (XmlNamespace ns : lNs) {
				namespaceURI.put(ns.getPrefix(), ns.getValue());
			}
		}catch(Throwable t){
			t.printStackTrace();
		}finally{
			aSession.close();
		}
	}
		
	/**
	 * Main method: perform the monitoring of all exchanges.
	 * @param exchange
	 */
	private void traceExchange(MessageExchange exchange) {
		
		debug("============================ Exchange ["+exchange.getExchangeId()+"] Properties  ================================== ");
		debug("MEP: " + exchange.getClass().getName());
		debug("Id & status: " + exchange.getExchangeId() + " " + exchange.getStatus());
		debug("Sender: "+exchange.getProperty(JbiConstants.SENDER_ENDPOINT));
		debug("Service: " + exchange.getService());
		debug("Endpoint: " + exchange.getEndpoint());
		debug("Role: " + exchange.getRole());
		debug("Interfacename: " + exchange.getInterfaceName());
		debug("Operation: " + exchange.getOperation());
		
		Session aSession = null;
		Transaction tx = null;
		try {
			JournalInstanceService jis = null;
			aSession = HibernateUtil.getSessionFactory().openSession();
			tx = aSession.beginTransaction();

			debug("Retrieve the MEP from Message Exchange ");
			String mep = getMep(exchange);
			debug("MEP Retrieved ["+mep+"]");

			// If the state of the exchange is ACTIVE we doesnt search for the
			// exchange on the database because IT CANNOT exist: it is the first exchange
			// of the transition
			if (!firstExchangeForTransition(exchange, mep)) {
				debug("Search Journal Instance Service if exist a record with exchangeID = "+exchange.getExchangeId());
				jis = AuditDBUtils.getJournalInstanceServiceByMsgExchangeId(aSession, exchange
						.getExchangeId());
			}
			
			// Retrieve correlation id to discover the process to which this
			// exchange belongs to
			NewExchangeStatus newExchangeStatus = null;
			boolean instanceSACreated = false;	// Setted to true if the instance of the service assembly
												// is created now: useful to limit the iter correlation work
												// only in this case
			
			if (jis != null) {
				// There is already a Message Exchange with this id logged: we
				// should update the existing
				// row with the state of exchange
				debug(" FOUND JIS -> Updating... " );
				newExchangeStatus = manageExistingMessageExchange(exchange, jis, mep, aSession);
				debug(" JIS Updated " );
			} else {
				// There isn't a Message Exchange with this id logged: we should
				// create a new one	
				debug(" JIS NOT FOUND -> Creating a new one.... " );
				JISCreationResult jisCreationResult = createMessageExchange(exchange, mep, aSession);
				jis = jisCreationResult.jis;
				instanceSACreated = jisCreationResult.instanceSACreated;
				debug((instanceSACreated ? " Created a new InstanceSA" : ""));
				
				newExchangeStatus = new NewExchangeStatus();
				if (jis != null) {
					newExchangeStatus.exchangeStatus = jis.getMepState();
				}
			}
			if (jis != null) {
				aSession.save(jis);

				// Retrieve the message to trace
				NormalizedMessage messageToTrace = retrieveMessageToTrace(exchange, mep);

				if (messageToTrace != null) {
					
					try {
						// Trace message
						LogJournal messageInfo = traceMessage(messageToTrace, jis, newExchangeStatus.exchangeStatus, aSession);
						
						// Trace message relevant data
						List<RelevantData> savedRelevantData = traceRelevantData(messageInfo, aSession);
						
						// Check if the instance of service assembly was just created (and not saved yet)
						// Consider ONLY at least one relavant data have been produced
						if ((instanceSACreated) && (savedRelevantData.size() > 0)) {
							
							checkIterCorrelation(messageInfo, savedRelevantData, aSession);
						}
					} catch (Exception ex) {
			        	log.error(" Exception during tracing of message ", ex);
			        	log.error(" Exception during tracing of message " + messageToTrace);
					}
					
				}
			}
			
			aSession.flush();
			tx.commit();
			
			// Verify if the process is terminated, only if necessary (only if
			// the current message exchange has status DONE or FAULT)
			if (newExchangeStatus.processFault) {
				manageFault(jis, aSession);
			} else if (newExchangeStatus.exchangeDone) {
				manageDone(jis, aSession);
			}
			
		} catch (Exception e) {
        	log.error("traceExchange ", e);
			tx.rollback();
		} finally {
			if (aSession != null) {
				aSession.close();
			}
		}
	}

	/**
	 * Check if this exchange is the first one for the current transition.
	 * @param exchange Exchange to check
	 * @param mep
	 * @return True if this exchange is the first one for the current transition.
	 */
	private boolean firstExchangeForTransition(MessageExchange exchange, String mep) {
		boolean firstExchangeForTransition = false;
		if (mep.equals(MEP_INONLY)) {
			debug(" MEP is IN_ONLY ");
			firstExchangeForTransition = exchange.getStatus().equals(ExchangeStatus.ACTIVE);
		} else if (mep.equals(MEP_INOUT)) {
			debug(" MEP is IN_OUT ");
			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
				// I must check if this is the request or the response
				firstExchangeForTransition = (exchange.getRole() == MessageExchange.Role.CONSUMER);
			}
		} else if (mep.equals(MEP_ROBUSTINONLY)) {
			debug(" MEP is ROBUST_IN_ONLY ");
		} else if (mep.equals(MEP_INOPTIONALOUT)) {
			debug(" MEP is ROBUST IN_OPTIONAL_OUT ");
		} else {
			// Errore
        	log.error("firstExchangeForTransition. Mep type=UNDEFINED ");
		}
		return firstExchangeForTransition;
	}
	
	/**
	 * Invoked when a FAULT message is received: it's responsible for updating the process state.
	 * @param jis Current JournalInstanceService
	 * @param aSession Hibernate session
	 */
	private void manageFault(JournalInstanceService jis, Session aSession) {
		Transaction finalTx = null;

		try {
			InstanceSa serviceAssemblyInstance = jis.getInstanceSa();
			
			// Check if the process state is ACTIVE yet
			if (serviceAssemblyInstance.getState() == ServiceAssemblyStateConstants.SA_ACTIVE) {
				// Begin a new transaction to close the process state
				finalTx = aSession.beginTransaction();

				// The process has faulted: update its status
				serviceAssemblyInstance.setState(ServiceAssemblyStateConstants.SA_FAULTED);
				/*Long idServiceAssemblySaved = (Long) */ aSession.save(serviceAssemblyInstance);
				aSession.flush();
				finalTx.commit();
			}
		} catch (Exception ex) {
        	log.error("manageFault ", ex);
        	log.error("manageFault, JournalInstanceService = " + jis);
			finalTx.rollback();
		}
	}
	
	/**
	 * Invoked when a DONE message is received: it's responsible for updating the process state.
	 * @param jis Current JournalInstanceService
	 * @param aSession Hibernate session
	 */
	private void manageDone(JournalInstanceService jis, Session aSession) {
		Transaction finalTx = null;

		try {
			InstanceSa serviceAssemblyInstance = jis.getInstanceSa();
			
			// Begin a new transaction to close the process state
			finalTx = aSession.beginTransaction();

			// If the exchange is done, we can remove it from database because
			// we dont need anymore the restart feature, for this exchange
			if (keepOnlyFaultedExchanges) {
				jis.setMessageExchange(null);
				aSession.save(jis);
			}
			
			// Check if the process state is ACTIVE yet
			if (serviceAssemblyInstance.getState() == ServiceAssemblyStateConstants.SA_ACTIVE) {

				long counter = AuditDBUtils.getJournalInstanceServiceNotDone(aSession, serviceAssemblyInstance);
				if (counter == 0) {
					// The process should be finished: update its status
					serviceAssemblyInstance.setState(ServiceAssemblyStateConstants.SA_DONE);
					/*Long idServiceAssemblySaved = (Long) */aSession.save(serviceAssemblyInstance);
				}
			}
			
			aSession.flush();
			finalTx.commit();
		} catch (Exception ex) {
        	log.error("manageDone ", ex);
        	log.error("manageDone, JournalInstanceService = " + jis);
			finalTx.rollback();
		}
	}
	
	/**
	 * Extracts ,from the current exchange, the message to trace on database: it can
	 * be the in message, the out message, the done message, etc
	 * @param exchange Current exchange
	 * @param mep Message Exchange Pattern
	 * @return NormalizedMessage to trace
	 */
	private NormalizedMessage retrieveMessageToTrace(MessageExchange exchange, String mep) {
		NormalizedMessage messageToTrace = null;
		if (mep.equals(MEP_INONLY)) {
			// The state can be only Active or Done
			debug(" MEP is IN_ONLY ");
			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE) ||
				exchange.getStatus().equals(ExchangeStatus.DONE)) {
				NormalizedMessage inMessage = ((InOnly) exchange).getInMessage();
				messageToTrace = inMessage;
			} else {
				// Error
				Fault faultDetail = ((InOnly) exchange).getFault();
				Exception ex = ((InOnly) exchange).getError();
	        	debug("retrieveMessageToTrace -> Fault = " + faultDetail);
	        	log.debug("retrieveMessageToTrace ->", ex);
	        	
	        	if (faultDetail != null) {
					messageToTrace = faultDetail;
				} else if (ex != null) {
					// Create a dummy message to allow storing the error
					try {
						NormalizedMessage errorMessage = exchange.createMessage();
						errorMessage.setContent(new StringSource((ex.getMessage() != null) ? ex.getMessage() : ex.toString()));
						messageToTrace = errorMessage;
					} catch (MessagingException mex) {
						log.error("Error creating error message", mex);
					}
				} else {
					log.error("No error to trace: both getFault and getError return null");
				}
			}
		} else if (mep.equals(MEP_INOUT)) {
			debug(" MEP is IN_OUT ");
			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
				
				// Check if there are errors: possible also if state is ACTIVE
				Fault faultDetail = ((InOut) exchange).getFault();
				if (faultDetail != null) {
					debug(" Message is FAULT");
					messageToTrace = faultDetail;
				}
				// I must check if this is the request or the response
				else if (exchange.getRole() == MessageExchange.Role.CONSUMER) {
					debug(" Role is CONSUMER ");
					NormalizedMessage outMessage = ((InOut) exchange).getInMessage();
					messageToTrace = outMessage;
				} else if (exchange.getRole() == MessageExchange.Role.PROVIDER) {
					debug(" Role is PROVIDER ");
					NormalizedMessage inMessage = ((InOut) exchange).getOutMessage();
					messageToTrace = inMessage;
				} else {
					// Error
		        	log.error("retrieveMessageToTrace. Mep type=MEP_INOUT, role unknown");
				}
			} else if (exchange.getStatus().equals(ExchangeStatus.DONE)) {
				NormalizedMessage inMessage = ((InOut) exchange).getInMessage();
				messageToTrace = inMessage;
			} else if (exchange.getStatus().equals(ExchangeStatus.ERROR)) {
				// Error
				Fault faultDetail = ((InOut) exchange).getFault();
				Exception ex = ((InOut) exchange).getError();
	        	debug("retrieveMessageToTrace -> Fault = " + faultDetail);
	        	log.debug("retrieveMessageToTrace ->", ex);
	        	
	        	if (faultDetail != null) {
					messageToTrace = faultDetail;
				} else if (ex != null) {
					// Create a dummy message to allow storing the error
					try {
						StringBuffer sb = new StringBuffer();
						sb.append("<MESSAGE SENT=\"FALSE\" ERROR=\"");
						sb.append(ex.getMessage());
						sb.append("\"/>");
						
						Fault fault = exchange.createFault();
						fault.setContent(new StringSource(sb.toString()));
						messageToTrace = fault;
					} catch (MessagingException mex) {
						log.error("Error creating error message", mex);
					}
				} else {
					log.error("No error to trace: both getFault and getError return null");
				}
			} else {
				// Error
	        	log.error("retrieveMessageToTrace. Mep type=MEP_INOUT, status undefined ");
			}
		} else if (mep.equals(MEP_ROBUSTINONLY)) {
			debug(" MEP is ROBUST_IN_ONLY ");
		} else if (mep.equals(MEP_INOPTIONALOUT)) {
			debug(" MEP is ROBUST IN_OPTIONAL_OUT ");
		} else {
			// Errore
        	log.error("retrieveMessageToTrace. Mep type=UNDEFINED ");
		}
		return messageToTrace;
	}
	
	/**
	 * Create on the database a new JournalInstanceService that corresponds to a new Message Exchange.
	 * @param exchange Current Message Exchange
	 * @param mep Message Exchange Pattern
	 * @param aSession Hibernate Session
	 * @return JISCreationResult result of the creation
	 */
	private JISCreationResult createMessageExchange(MessageExchange exchange, String mep, Session aSession) {
		// There isn't a Message Exchange with this id logged: we should
		// create a new one
		debug("createMessageExchange:: START");
		JISCreationResult result = new JISCreationResult();
		
		// Retrieve exchange correlation id to discover the process to
		// which the exchange
		// belongs to
		String correlationID = (String) exchange.getProperty(JbiConstants.CORRELATION_ID);
		debug("createMessageExchange:: Correlation ID ["+correlationID+"]");
		InstanceSa serviceAssemblyInstance = AuditDBUtils.getInstanceSaByIdCorrelation(aSession, correlationID);

		
		Endpoint sourceEndPoint = null;
		Endpoint targetEndPoint = null;
		
		// Retrieve SOURCE service for the exchange
		// Retrieve SOURCE service endpoint: service name (QName) + endpoint name
		String sourceServiceEndpointName = (String)exchange.getProperty(JbiConstants.SENDER_ENDPOINT);
		debug("createMessageExchange -> Sender Endpoint ["+sourceServiceEndpointName+"]");
		if (sourceServiceEndpointName == null) {
			// Errore
	    	log.error("createMessageExchange -> No configured Sender endpoint for exchange:" + exchange);
		}
		QName sourceServiceQName = QName.valueOf(sourceServiceEndpointName);
		String sourceServiceName = sourceServiceQName.getLocalPart();
		sourceServiceName = sourceServiceName.substring(sourceServiceName.indexOf(':') + 1);
		sourceEndPoint = AuditDBUtils.getEndpointByName(aSession, sourceServiceQName.getNamespaceURI(), sourceServiceName);
		
		if (sourceEndPoint == null) {
			debug("Process not registered on database, sourceEndPoint " + sourceEndPoint + " not found");
			return result;
		}

		// Retrieve TARGET service for the exchange
		QName targetServiceName = exchange.getService();
		targetEndPoint = AuditDBUtils.getEndpointByName(aSession, targetServiceName.getNamespaceURI(), targetServiceName.getLocalPart());
		debug("createMessageExchange -> Target Endpoint ["+targetEndPoint+"]");

		if (targetEndPoint == null) {
			debug("Process not registered on database, targetEndPoint " + targetEndPoint + " not found");
			debug("targetServiceName.getNamespaceURI(): " + targetServiceName.getNamespaceURI());
			debug("targetServiceName.getLocalPart(): " + targetServiceName.getLocalPart());
			return result;
		}
		
		if (serviceAssemblyInstance == null) {
			// This is a new process instance: create a new row
			// Before creating a new service assembly instance, I need
			// to retrieve
			// the service assembly, and to retrieve it I can use the
			// endpoint
			serviceAssemblyInstance = createServiceAssemblyInstance(targetEndPoint.getServiceAssembly(), correlationID, aSession);
			result.instanceSACreated = true;
		}

		// The instance of the process already exists:
		// we have to create the journal_instance_service row
		JournalInstanceService jis = new JournalInstanceService();
		jis.setInstanceSa(serviceAssemblyInstance);
		jis.setIdMsgExchange(exchange.getExchangeId());
		jis.setExchangeStart(new Date());

		MepState mepState = null;
		if (mep.equals(MEP_INONLY)) {
			mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INONLY_ACTIVE);
		} else if (mep.equals(MEP_ROBUSTINONLY)) {
			mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_ROBUSTINONLY_ACTIVE);
		} else if (mep.equals(MEP_INOUT)) {
			mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_ACTIVE);
		} else if (mep.equals(MEP_INOPTIONALOUT)) {
			mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOPTIONALOUT_ACTIVE);
		}
		jis.setMepState(mepState);
		jis.setEndpointByIdEndpointSource(sourceEndPoint);
		jis.setEndpointByIdEndpointTarget(targetEndPoint);
		
		// Store Message Exchange for allow restart in case of errors
		if (storeExchangesForRestart) {
			jis.setMessageExchange(serializeExchange(exchange));
		}
		
		result.jis = jis;
		return result;
	}
	
	/**
	 * Serializes an exchange on a byte array
	 * @param exchange Exchange to serialize
	 * @return Serialized exchange
	 */
	private byte[] serializeExchange(MessageExchange exchange) {
        byte[] buf = null;
        
        // Serialize to a byte array
		try {
	        ByteArrayOutputStream bos = new ByteArrayOutputStream();
	        ObjectOutputStream out = new ObjectOutputStream(bos);
	        out.writeObject(exchange);
	        out.close();
	    
	        // Get the bytes of the serialized object
	        buf = bos.toByteArray();
		} catch (Exception ex) {
			log.error("Error serializing exchange " + exchange, ex );
		}
        return buf;
	}

	/*
	 * NOTE:
	 * Developing with servicemix we encounter these possibility to notify error by components:
	 * 1 - maintain the Active status and provide the fault message
	 * 2 - set the Error status and provide an error message (+ fault message?)
	 * 
	 * Is not clear if point 1 is JBI compliant.
	 * 
	 */
	private NewExchangeStatus manageExistingMessageExchange(MessageExchange exchange, JournalInstanceService jis, String mep, Session aSession) {
		// There is already a Message Exchange with this id logged: we
		// should update the existing
		// row with the state of exchange
		
		NewExchangeStatus newExchangeStatus = new NewExchangeStatus();

		if (mep.equals(MEP_INONLY)) {
			// The state can be only Done
			if (exchange.getStatus().equals(ExchangeStatus.DONE) && (exchange.getError() == null)) {
				// Exchange terminated
				MepState mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INONLY_DONE);
				jis.setMepState(mepState);
				
				// Mark the exchange as done, to check if the process is terminated
				newExchangeStatus.exchangeDone = true;
			} else {
				// Exchange terminated with ERROR: this MEP state is not a standard JBI state
				// We introduced this state to manage properly the error case with InOnly MEP
				MepState mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INONLY_FAULT);
				jis.setMepState(mepState);
				
				// Mark the exchange as faulted, to check if the process is terminated
				newExchangeStatus.processFault = true;
			}
			newExchangeStatus.exchangeStatus = jis.getMepState();
		} else if (mep.equals(MEP_INOUT)) {

			NormalizedMessage faultMessage = ((InOut) exchange).getFault();
			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
				MepState mepState = null;
				
				if (faultMessage == null) {
					NormalizedMessage outMessage = ((InOut) exchange).getOutMessage();
					if (outMessage == null) {
						// Errore
						log.error("Both Out and Fault messages empty");
					}
					mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_OUT);
				} else {
					// Fault
					mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_FAULT);
				}
				jis.setMepState(mepState);
				newExchangeStatus.exchangeStatus = jis.getMepState();

			} else if (exchange.getStatus().equals(ExchangeStatus.DONE)) {
//				if (faultMessage != null) {
//					// error
//					MepState mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_FAULT);
//					jis.setMepState(mepState);
//					newExchangeStatus.processFault = true;
//				} else {

				// If there are errors, we dont mark the process as DONE in the database, so it's visualized
				// with an error state on the console
				if (faultMessage == null) {
					MepState mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_DONE);
					jis.setMepState(mepState);
				} else {
					// Keep previous state: should be FAULT
				}
				
				// Real exchange status is DONE
				newExchangeStatus.exchangeStatus = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_DONE);
				
				// Mark the exchange as done, to check if the process is terminated
				newExchangeStatus.exchangeDone = true;
//				}
			} else {
				// Error
				MepState mepState = (MepState) aSession.get(MepState.class, MepStateConstants.MEP_INOUT_FAULT);
				jis.setMepState(mepState);
				
				newExchangeStatus.exchangeStatus = jis.getMepState();
				newExchangeStatus.processFault = true;
			}

		} else if (mep.equals(MEP_ROBUSTINONLY)) {
			// Not handled yet
		} else if (mep.equals(MEP_INOPTIONALOUT)) {
			// Not handled yet
		} else {
			// Errore
			log.error("MEP not supported: " + mep);
		}
		return newExchangeStatus;
	}

	public String getMep(MessageExchange exchange) {
		String mepClassName = exchange.getClass().getName();
		return mepClassName;
	}

	private LogJournal traceMessage(NormalizedMessage message, JournalInstanceService journalInstanceService, MepState exchangeState, Session aSession) 
		throws Exception {
		SourceTransformer sourceTransformer = new SourceTransformer();
		String xmlMessage = sourceTransformer.contentToString(message);

		LogJournal messageInfo = new LogJournal();
		messageInfo.setJournalInstanceService(journalInstanceService);
		messageInfo.setMepState(exchangeState);
		messageInfo.setDatelog(new java.util.Date());
		// Recreate the content again, if there was a content
		message.setContent((xmlMessage != null) ? new StringSource(xmlMessage) : null);
		messageInfo.setLog(xmlMessage);
		
		aSession.save(messageInfo);
		
		return messageInfo;
	}
	
	/**
	 * Trace relevant data for current message
	 * @param messageInfo Current message
	 * @param aSession Hibernate session
	 * @return List of RelevantData
	 * @throws Exception
	 */
	private List<RelevantData> traceRelevantData(LogJournal messageInfo, Session aSession) 
		throws Exception {
		// 1. Retrieve all rules configured for current endpoint
		// The rules should be ordered by order_id
		// 2. Filter valid rules by date
		// 3. Apply each rule to produce relevant data
		
		List<RelevantData> result = new ArrayList<RelevantData>();
		
		// Retrieve current target endpoint
		JournalInstanceService jis = messageInfo.getJournalInstanceService();
		Endpoint targetEndpoint = jis.getEndpointByIdEndpointTarget();
		// Retrieve rules for target endpoint: the rules are
		// already filtered by date
		debug("traceRelevantData -> Get Rules For Enpoint ID ["+ targetEndpoint.getIdEndpoint()+"] name ["+targetEndpoint.getName()+"]" );
		Collection<Rule> rules = AuditDBUtils.getRulesByEndpoint(aSession, targetEndpoint, messageInfo.getMepState());
		
		debug("traceRelevantData -> Found ["+rules.size()+"] rules for Endpoint with id ["+ targetEndpoint.getIdEndpoint()+"] and name ["+targetEndpoint.getName()+"]" );
		// 3. Apply each rule to produce relevant data
		for (Iterator it = rules.iterator(); it.hasNext(); ) {
			Rule currentRule = (Rule)it.next();
			
			// Create a DOM node from XML text
			Document document = DocumentHelper.parseText(messageInfo.getLog());
			//Node node = document.selectSingleNode(currentRule.getExpr());
			
			XPath xPathExpression = new DefaultXPath(currentRule.getExpr());
			xPathExpression.setNamespaceURIs(getNamespaceURI());
			String extractedData = xPathExpression.valueOf(document);
			debug("-> Applying Rule id["+currentRule.getIdRule()+"] Xpath -> "+ currentRule.getExpr());
			debug("-> To message " + messageInfo.getLog() + "");
			
			

			if ((extractedData == null) || (extractedData.trim().length() == 0)){
				//
				// NON GENERO UN'ECCEZZIONE 
				// Semplicemente non salvo i dati
				//
				//throw new IllegalStateException(" No data extracted.");
				log.warn("->Rule ["+ currentRule.getExpr() + "] Have Produced No Data");
			}
			debug("-> Rule ["+ currentRule.getExpr() + "] extract Extracted Data ["+extractedData+"]");
			RelevantData relevantData = new RelevantData();
			relevantData.setLogJournal(messageInfo);
			relevantData.setValue(extractedData);
			relevantData.setAttribute(currentRule.getAttribute());
			
			aSession.save(relevantData);
			aSession.flush();
			
			InstanceSa instanceSA = jis.getInstanceSa();
			RelevantDataInstanceSa relevantDataInstanceSa = 
				new RelevantDataInstanceSa(new RelevantDataInstanceSaId(instanceSA.getIdInstanceSa(), relevantData.getIdRelevantData()),
						relevantData, instanceSA);
			aSession.save(relevantDataInstanceSa);
			
			result.add(relevantData);
			
			// If there is an external feeder configured, invoke it
			if (currentRule.getExternalDataFeeder() != null) {
//				IDataFeeder externalFeeder = DataFeederFactory.getInstance(aSession).getFeeder(aSession, currentRule.getExternalDataFeeder());
//				externalFeeder.traceData(aSession, relevantData);
			}
		}
		return result;
	}
	
	private InstanceSa createServiceAssemblyInstance(it.eng.spagosoa.metadb.model.ServiceAssembly serviceAssembly, String correlationID, Session aSession) {
		// This is a new process instance: create a new row
		// Before creating a new service assembly instance, I need
		// to retrieve
		// the service assembly, and to retrieve it I can use the
		// endpoint

		InstanceSa serviceAssemblyInstance = new InstanceSa();
		serviceAssemblyInstance.setServiceAssembly(serviceAssembly);
		serviceAssemblyInstance.setStartdate(new Date());
		serviceAssemblyInstance.setEnddate(new Date());
		serviceAssemblyInstance.setIdCorrelation(correlationID);
		serviceAssemblyInstance.setState(ServiceAssemblyStateConstants.SA_ACTIVE);

		Long idServiceAssemblySaved = (Long) aSession.save(serviceAssemblyInstance);
		serviceAssemblyInstance = (InstanceSa) aSession.get(InstanceSa.class, idServiceAssemblySaved);
		return serviceAssemblyInstance;
	}
	
	public void checkIterCorrelation(LogJournal messageInfo, List<RelevantData> savedRelevantData, Session aSession) throws Exception {
		
		JournalInstanceService jis = messageInfo.getJournalInstanceService();
		InstanceSa instanceSA = jis.getInstanceSa();
		ServiceAssembly serviceAssembly = instanceSA.getServiceAssembly();
		
		// Get all the iter type that have in the definition the Service Assembly of this instance
		List<IterType> candidateIterTypes = AuditDBUtils.getCandidateIterTypesForServiceAssembly(serviceAssembly, aSession);
		
		String saName = instanceSA.getServiceAssembly().getName();
		debug("checkIterTypeCorrelation -> Get the list of the iter types that the ["+saName+"] belongs to");
		
		IterType theIterType = null;
		if (candidateIterTypes.size() > 0){
			if (candidateIterTypes.size() == 1){
				debug("checkIterTypeCorrelation -> Process ["+ saName +"] have only one iter type as candidate iter");
				theIterType = candidateIterTypes.get(0);
				debug("checkIterTypeCorrelation -> There's no need to apply rules the iter type is ["+theIterType.getName()+"]");
			}else{
				for (IterType i : candidateIterTypes){
					debug("checkIterTypeCorrelation -> Process ["+ saName +"] is part of Iter Type ["+i.getName()+"]");
			
					if ( belongsToIterType(messageInfo, i)){
						// The rule is verified 
						theIterType = i;
						break;
					}
				}
			}
		}else{
			debug("checkIterTypeCorrelation -> Process ["+ saName +"] have no candidate iter");
		}
		
		
		
		if (theIterType == null){
			debug("checkIterTypeCorrelation -> Iter Type not found");
			return;
		}else{
			debug("checkIterTypeCorrelation -> Found the Iter Type ["+theIterType.getName()+"]");
		}
		
		// We have the iterType
		
		if (AuditDBUtils.isFirstServiceAssemblyInIterType(serviceAssembly, theIterType, aSession)){
				// New iter instance
				debug("checkIterTypeCorrelation -> Process["+serviceAssembly.getName()+"] is the first in Iter Type ["+theIterType.getName()+"] Defnition");
				debug("checkIterTypeCorrelation -> Try to create a new Iter Instance...");
				newIterInstance(instanceSA, theIterType,savedRelevantData,aSession);
				debug("checkIterTypeCorrelation -> Iter type created...");
		} else {
				// Existing iter instance
				debug("checkIterTypeCorrelation -> Process["+serviceAssembly.getName()+"] is the first in Iter Type ["+theIterType.getName()+"] Defnition");
				debug("checkIterTypeCorrelation -> Try to Relate this instance to another iter instance...");
				existingIterInstance(instanceSA, theIterType, savedRelevantData, aSession);
				debug("checkIterTypeCorrelation -> Iter type correlation finished");
		}

	}
	
	/**
	 * Checks if a message belongs to an existing iter type
	 * @param messageInfo Current message
	 * @param iterType Iter type to check
	 * @return True if the message belongs to the iter type
	 */
	public boolean belongsToIterType(LogJournal messageInfo, IterType iterType ){
		boolean result = false;
		try{
			String booleanXPathExpression = iterType.getRule().getExpr();
			debug("belongsToIterType -> Verify if the rule ["+booleanXPathExpression+"] applies to the message");
			debug(messageInfo.getLog());
			Document document = DocumentHelper.parseText(messageInfo.getLog());
			//DefaultXPath aDefaultXPath = new DefaultXPath(xPathExpression);
			//result =  aDefaultXPath.booleanValueOf(document);
			XPath xPathExpression = new DefaultXPath(booleanXPathExpression);
			xPathExpression.setNamespaceURIs(getNamespaceURI());
			result =  xPathExpression.booleanValueOf(document);
			debug("belongsToIterType -> Rule ["+booleanXPathExpression+"]" + ( result ? "Is Verified" : "Is Not Verified"));
		}catch (Exception ex) {
			log.error("belongsToIterType -> Exception occured ", ex );
			result = false;
		}
		
		return result;
	}
	
	
	/**
	 * Creates a new iter instance on database
	 * @param instanceSA Service Assembly instance
	 * @param iterType Iter type of the iter instance to create
	 * @param savedRelevantDataList List of relevant data the compose the iter unique key
	 * @param aSession Hibernate session
	 */
	private void newIterInstance(InstanceSa instanceSA, IterType iterType, List<RelevantData> savedRelevantDataList, Session aSession) {
		// New iter instance
		// We have to create a new iter type instance and link it to this process instance
		InstanceIter iterInstance = new InstanceIter();
		
		Set<InstanceSa> processInstances = new LinkedHashSet<InstanceSa>(1);
		processInstances.add(instanceSA);
		iterInstance.setInstanceSas(processInstances);
		iterInstance.setIterType(iterType);
		
		String uniqueIterInstanceKey = computeUniqueIterInstanceKey(iterType, savedRelevantDataList,aSession );
		if (uniqueIterInstanceKey == null){
			log.error("newIterInstance - > Cannot Create Iter Instance");
			return;
		}
		iterInstance.setUniqueIterInstanceKey(uniqueIterInstanceKey);
		aSession.save(iterInstance); 
		// Associate the Service Assembly instance to the iter instance
		instanceSA.setInstanceIter(iterInstance);
		aSession.save(instanceSA);
	
	}
	
	/**
	 * Computes a unique key for an iter, starting from a list of relevant data
	 * @param iterType Iter type of the iter instance
	 * @param savedRelevantDataList List of relevant data
	 * @param aSession Hibernate session
	 * @return The unique key for iter
	 */
	private String computeUniqueIterInstanceKey(IterType iterType,  List<RelevantData> savedRelevantDataList, Session aSession){
		List<Attribute> attributesForIter = AuditDBUtils.getAttributeForIterType(iterType, aSession);
		StringBuffer sb = new StringBuffer();
		for (Attribute attribute : attributesForIter){
			boolean found = false;
			
			for (RelevantData rd : savedRelevantDataList){
				if (rd.getAttribute().getIdAttribute() == attribute.getIdAttribute()){
					sb.append("["+attribute.getIdAttribute()+":"+rd.getValue()+"]");
					found = true;
					break;
				}
			}
			if (!found)
				return null;
		}
		return ( sb.length() > 0 ? sb.toString() : null );
		
	}
	private void existingIterInstance(InstanceSa instanceSA, IterType iterType, List<RelevantData> savedRelevantDataList, Session aSession) 
		throws Exception {
		
		String iterInstanceKey = computeUniqueIterInstanceKey(iterType, savedRelevantDataList, aSession);
		
		if (iterInstanceKey == null){
			log.error("existingIterInstance -> This instance sa doe not belong to iter instance ");
			return;
		}
		
		InstanceIter istanceIter = AuditDBUtils.getInstanceIterByUniqueKey(iterInstanceKey, iterType, aSession );
		if (istanceIter == null){
			log.error("existingIterInstance -> This instance sa doe not belong to iter instance ");
			return;
		}
		
		//Now link the iter instance to the service assembly instance
		instanceSA.setInstanceIter(istanceIter);
		aSession.save(instanceSA);
	}
	
    /**
     * Loads the configuration for the management of exchanges for restart, from a properties
     * file stored inside the SU.
     * 
     */
    protected void loadRestartConfiguration() {
        String uri = "META-INF/monitorConfiguration";
        try {
            InputStream in = loadResource(uri);
            Properties props = new Properties();
            props.load(in);
            
        	storeExchangesForRestart = Boolean.parseBoolean(props.getProperty("storeExchangesForRestart"));
        	keepOnlyFaultedExchanges = Boolean.parseBoolean(props.getProperty("keepOnlyFaultedExchanges"));
        } catch (IOException e) {
            log.error("Could not find resource: " + uri);
        }
    }
    
    
    /**
     * Loads the resource from the given URI
     */
    protected InputStream loadResource(String uri) {
        // lets try the thread context class loader first
        InputStream in = Thread.currentThread().getContextClassLoader().getResourceAsStream(uri);
        if (in == null) {
            in = getClass().getClassLoader().getResourceAsStream(uri);
            if (in == null) {
                debug("Could not find resource: " + uri);
            }
        }
        return in;
    }
	
		
}
