/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.talend;

import java.lang.reflect.Method;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.transform.Source;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.StringSource;

public class TalendCaller extends TransformComponentSupport{

	private static final Log log = LogFactory.getLog(TalendCaller.class);
	private Class talendJobClass = null; 
	private String  talendJobClassName = null;
	private Method mainMethod = null;
	
	
	
	public void start() throws JBIException {
    	
		try{
			log.debug("Talend Caller::start() -> Starting Taleb Caller with Job Class ["+talendJobClassName+"]");
			this.talendJobClass = Class.forName(talendJobClassName);
			this.mainMethod = this.talendJobClass.getMethod("main", new Class[]{String[].class});
			log.debug("Talend Caller::start() -> Talend Job ["+talendJobClassName+"] started ");
			
		}catch (Exception e) {
			log.error("Talend Caller::start() ->   Transformer Class Istantiation Failed");
			log.error(e);
			throw new JBIException(e.getMessage());
		}
    }
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {
		try{	
			
			Object[] mainMethodParams  = new Object[1];
			String[] args = new String[0];
			mainMethodParams[0] = args;
 			mainMethod.invoke(null, mainMethodParams);
			Source outMsg = null;
			outMsg = toXMLSource("<OK>Talend Caller is OK </OK>");
			if (outMsg != null) {
	              
	                out.setContent(outMsg);
			}else{
				Fault fault = exchange.createFault();
	        	fault.setContent(new StringSource("<ERROR>Out Msg is null </ERROR>"));
	        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
	        	throw new FaultException("Talend Caller Failed ", exchange, fault);
			}
			return true;
		}catch (Throwable t) {
    		log.error("Exception in Talend Caller");
    		log.error(t);
    		Fault fault = exchange.createFault();
        	fault.setContent(new StringSource("<ERROR>An error is occured in Talend Caller</ERROR>"));
        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
        	throw new FaultException("Talend Caller Failed ", exchange, fault);
		}
			
	}
	
	protected Source toXMLSource(String s) throws Exception {
        return new StringSource(s);
    }

	public String getTalendJobClassName() {
		return talendJobClassName;
	}

	public void setTalendJobClassName(String talendJobClassName) {
		this.talendJobClassName = talendJobClassName;
	}
	
    
	
}