/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.tcp;


import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.mina.common.ByteBuffer;
import org.apache.mina.common.IoSession;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.components.util.ComponentSupport;

public class TCPServerInOutBinding extends ComponentSupport {
	private static final Log log = LogFactory.getLog(TCPServerInOutBinding.class);
	private TCPConsumerEndpoint endpoint;
	private TCPMarshaller marshaller;
	
	public TCPServerInOutBinding(TCPConsumerEndpoint endP) throws JBIException {
		endpoint = endP;
		super.init(endpoint.getContext());
	}
	
	private TCPMarshaller getMarshaller() {
		if (marshaller == null) {
			marshaller = new TCPMarshaller(endpoint.getConfig());
		}
		return marshaller;
	}
	
	@Override
	public void start() throws JBIException {
	}
	
	private boolean isInOutMep() {
		return endpoint.getConfig().getDefaultMep().toString().equals(CommonConfig.DEFINOUTMEP);
	}
	
	private boolean isInOnlyMep() {
		return endpoint.getConfig().getDefaultMep().toString().equals(CommonConfig.DEFINMEP);
	}
	
	public void process(IoSession session, ByteBuffer msg) throws JBIException {
		log.debug("process: start");
		// you have to:
		// 1 - Log data (take a look at what you can do with LoggerFilter)
		// 2 - Handle message and marshar in a message exchage to send to NMR
        MessageExchangeFactory factory = getExchangeFactory();
        MessageExchange exchange = null;
        if (isInOutMep()) {
        	exchange = factory.createInOutExchange();
        }
        if (isInOnlyMep()) {
        	exchange = factory.createInOnlyExchange();
        }
        configureExchangeTarget(exchange);
        NormalizedMessage inMessage = exchange.createMessage();
        try {
        	// set on inMessage marshalled message
        	getMarshaller().toNMS(exchange, inMessage, msg);
        	if (isInAndOut(exchange)) {
        		((InOut)exchange).setInMessage(inMessage);
        	} else {
        		((InOnly)exchange).setInMessage(inMessage);
        	}
            
            // TODO check if the send has to be synchronous or not
            if ( CommonConfig.OPERATION_MODE_BIDIRECTIONAL.equals(
            		endpoint.getConfig().getPointMode()) ) {
            	getDeliveryChannel().send(exchange);
            } else {
            	boolean result = getDeliveryChannel().sendSync(exchange);
            	
	            if (result) {
	            	if (exchange.getStatus() == ExchangeStatus.ERROR) {
	            		if (exchange.getError() != null) {
	            			// comunicate a specific error?
	            		} else {
	            			// comunicate a generic error
	            		}
	            	}
	            }
            }
            String correlationId = 
            	(String)exchange.getProperty(JbiConstants.CORRELATION_ID);
        	endpoint.addCorrelatedSession(correlationId, session);
	    }
	    finally {
			log.debug("process: end");
	    }		
	}
	
	private void configureExchangeTarget(MessageExchange exchange) {
		QName targetService = endpoint.getTargetService();
		String targetEndpoint = endpoint.getTargetEndpoint();
        exchange.setService(targetService);
        if (targetEndpoint != null) {
            ServiceEndpoint se = getContext().getEndpoint(targetService, targetEndpoint);
            if (se != null) {
                exchange.setEndpoint(se);
            } else {
                logger.warn("Target service (" + targetService + ") and endpoint (" + targetEndpoint + ")" + 
                        " specified, but no matching endpoint found.  Only the service will be used for routing.");
            }
        }
	}
}
