/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.servicemix.drools.model;

import java.math.BigDecimal;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.naming.Context;
import javax.sql.DataSource;

import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.handlers.MapListHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author: Andrea Zoppello
 */
public class DbHelper {
    private static final Log log = LogFactory.getLog(DbHelper.class);
    private Context aContext = null;
//    private static final String jndiInitialContextFactory = "org.apache.xbean.spring.jndi.SpringInitialContextFactory";
//    private static final String jndiProviderUrl = "classpath:jndi.xml";
        
    public DbHelper(Context context){
    	this.aContext = context;
    	/*
    	try{
    		Hashtable env = new Hashtable();
    		env.put(Context.INITIAL_CONTEXT_FACTORY, jndiInitialContextFactory);
    		env.put(Context.PROVIDER_URL, jndiProviderUrl);

    		aContext = new InitialContext(env);
    		 
    	}catch (NamingException e) {
    		
			log.error("Exception is occurred In InitialContext");
			log.error(e);
		
    	}*/
    }

    public int count(DataSource ds, String sqlQuery, Object[] params){
       Connection aConnection = null;
       try{
           log.debug(" Try to get Connection ");
           aConnection = ds.getConnection();
           log.debug(" Set autocommit false ");
           aConnection.setAutoCommit(false);
           QueryRunner runQuery = new QueryRunner();
           log.debug(" Executing Query ["+sqlQuery+"]");
           for (int i=0; i < params.length; i++){
               log.debug(" Params["+i+"] ["+params[i]+"]");
           }
           List res = (List) runQuery.query(aConnection,sqlQuery,params,new MapListHandler());
           log.debug(" Result Size = ["+res.size()+"]");
           return res.size();
          
        }catch(Throwable e){
           log.error("Exception is occurred Executing Query ");
           log.error(e);
           
           return -1;
        }finally{
            DbUtils.rollbackAndCloseQuietly(aConnection);
        }
        
    }
    
    public String getQueryString(String tableName, String fieldName){
       String sql = "SELECT T."+fieldName+" FROM "+tableName+" T WHERE T."+fieldName+" = ? ";
       return sql;
    }
    
    public Object[] getParams(String value, String valueType){
       Object[] params = new Object[1];
       if (valueType.equalsIgnoreCase("DECIMAL")){
          params[0] = new BigDecimal(value);
       }else  if (valueType.equalsIgnoreCase("INT")){
         params[0] = Integer.valueOf(value);
       }else if (valueType.equalsIgnoreCase("LONG")){
         params[0] = Long.valueOf(value);
       }else if (valueType.startsWith("DATE")){
         String dateFormat = valueType.substring("DATE(".length() + 1, valueType.length() - 1);
         log.debug(" Parsing ["+value+"] AS DATE WITH FORMAT ["+dateFormat+"]");
         try{
           SimpleDateFormat s = new SimpleDateFormat(dateFormat);
           Date d = s.parse(value);
           params[0] = new java.sql.Date(d.getTime());
         }catch(Exception e ){
          log.error("Exception " + e );
         }
       }else{
         params[0] = value;
       }
       return params;
     }
    
    public boolean exist(String value, String valueType, String tableName, String fieldName,String dsName){
    	log.debug(" DbHelper exist start ");
    	log.debug(" exist("+value+", "+valueType+", "+tableName+", "+fieldName+","+dsName+")");
    	int nRec = count(getDataSource(dsName), getQueryString(tableName, fieldName), getParams(value, valueType));
    	log.debug(" exist return nRec ["+nRec+"]");
    	return nRec > 0;
    }
    
    public boolean existOne(String value, String valueType, String tableName, String fieldName,String dsName){
       log.debug(" DbHelper existOne ");
       log.debug(" existOne("+value+", "+valueType+", "+tableName+", "+fieldName+","+dsName+")");
       int nRec = count(getDataSource(dsName), getQueryString(tableName, fieldName), getParams(value, valueType));
       log.debug(" existOne return nRec ["+nRec+"]");
       return nRec == 1;
    }
    
    public DataSource getDataSource(String dsName){
     
        DataSource ds = null;
        String completeJndiDataSourceName = "java:comp/env/jdbc/" + dsName;
        try{
        	log.debug(" Looking Up Context For DataSource ["+completeJndiDataSourceName+"]");
            ds = (DataSource)aContext.lookup(completeJndiDataSourceName);
            if (ds == null){
            	log.debug(" Looking Up FAILS Return a Null DataSource ");
            }else{
            	log.debug(" Looking Up OK ");
            }
        }catch(javax.naming.NamingException ne){
        	log.error("Exception occurrend Looking up for DataSource ");
        	log.error(ne);
        }
        return ds;
    }

    
    
}

